#include "GSPlayer2.h"
#include "KayMainWnd.h"
#include "..\Win32\Win32Options.h"

CKayMainWnd::CKayMainWnd()
{
	m_fVolMuted = FALSE;
	RepeatLoopReset();
}

CKayMainWnd::~CKayMainWnd()
{
}

void CKayMainWnd::RepeatLoopReset(BOOL fDisplay /*= FALSE*/)
{
//	if(m_hWnd && IsWindow(m_hWnd)) {
//		KillTimer(m_hWnd, ID_TIMER_REPEAT_LOOP);
//		if(fDisplay)
//			SetTimer(m_hWnd, ID_TIMER_REPEAT_LOOP, TIMER_REPEAT_LOOP_INTERVAL, NULL);
//	}
	m_fDrawRepeatLoop = fDisplay;
	m_nRepeatLoopState = 0;
	m_lRepeatLoopStart = -1;
	m_lRepeatLoopEnd = -1;
	m_lRepeatLoop_A = -1;
	m_lRepeatLoop_B = -1;
}

//void CKayMainWnd::OnCreate(HWND hWnd)
//{
//	CMainWnd::OnCreate(hWnd);
//	ChangeTitle(_T("KayPlayer"));
//}

LRESULT CKayMainWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)) {
		case IDM_APP_EXIT:
			PostMessage(m_hWnd, WM_CLOSE, 0, 0);
			return 0;
		case IDM_TOOL_VOLMUTE:
			OnVolumeMute();
			return 0;
		case IDM_PLAY_NEXT_FOLDER:
			OnPlayNextFolder();
			return 0;
		case IDM_PLAY_PREV_FOLDER:
			OnPlayPrevFolder();
			return 0;
		case IDM_REPEAT_LOOP:
		case IDM_REPEAT_LOOP_CLEAR:
		case IDM_REPEAT_LOOP_SET_START:
		case IDM_REPEAT_LOOP_SET_END:
			OnRepeatLoop(LOWORD(wParam));
			return 0;
	}
	return CMainWnd::OnCommand(wParam, lParam);
}


void CKayMainWnd::OnVolumeMute()
{
	m_fVolMuted = !m_fVolMuted;
	InitVolume();
	m_fDrawVolume = TRUE;
	UpdateTitle();
}

void CKayMainWnd::OnToolOption()
{
	CMainWnd::OnToolOption();
	InitVolume();
}

void CKayMainWnd::OnPlayRepeat()
{
	// ٲٱ  override
	switch (m_pOptions->m_fRepeat) {
	case COptions::RepeatNone:
		m_pOptions->m_fRepeat = COptions::RepeatOne;
		break;
	case COptions::RepeatOne:
		m_pOptions->m_fRepeat = COptions::RepeatAll;
		break;
	default:
		m_pOptions->m_fRepeat = COptions::RepeatNone;
		break;
	}
	UpdatePlayOption();
}

void CKayMainWnd::ChangeTitle(LPTSTR str)
{
	if( IsWindow(m_hWnd) )
		SetWindowText(m_hWnd, str);
}

void CKayMainWnd::InitVolume()
{
	if(m_fVolMuted) {
		MAP_SetVolume(m_hMap, 0);//Mute Volume
	}
	else if(m_pOptions->m_fUseSystemVolume) {
		MAP_SetVolume(m_hMap, (DWORD)-1);//MAX Volume
	}
	else {
		MAP_SetVolume(m_hMap, m_dwCurrentSoftVolume);
	}
}

BOOL CKayMainWnd::Open(int nIndex)
{
	InitVolume();
	RepeatLoopReset();

	return CMainWnd::Open(nIndex);
}

MMRESULT CKayMainWnd::waveOutGetVolume(HWAVEOUT hwo, LPDWORD pdwVolume)
{
	MMRESULT res = MMSYSERR_NOERROR;
	if(m_pOptions->m_fUseSystemVolume) {
		res = ::waveOutGetVolume(NULL, pdwVolume);
	} else {
		*pdwVolume = m_dwCurrentSoftVolume;
	}
	return res;
}

MMRESULT CKayMainWnd::waveOutSetVolume(HWAVEOUT hwo, DWORD dwVolume)
{
	MMRESULT res = MMSYSERR_NOERROR;
	m_dwCurrentVolume = dwVolume;

	//  ̹ Ʈ ´.
	//

	//  ý  Ʈ Ѵ
	if(m_pOptions->m_fUseSystemVolume) {
		res = ::waveOutSetVolume(NULL, dwVolume);
	}
	else {//Ʈ  ΰ
		m_dwCurrentSoftVolume = dwVolume;
		// 尡 ƴҶ Ѵ.
		if(!m_fVolMuted) {
			MAP_SetVolume(m_hMap, dwVolume);
		}
	}
	return res;
}

void CKayMainWnd::OnTimer(UINT uID)
{
	if (uID == ID_TIMER_TIME) {
		if( (m_nRepeatLoopState&0x03) == 0x03 ) {
			long current = MAP_GetCurrent(m_hMap);
			if (current >= m_lRepeatLoopEnd && m_lRepeatLoopEnd!=-1 && m_lRepeatLoopStart!=-1) {
				MAP_Seek(m_hMap, m_lRepeatLoopStart);
			}
		}

		UpdateTime();
	}
	else if (uID == ID_TIMER_REPEAT_LOOP) {
		KillTimer(m_hWnd, ID_TIMER_REPEAT_LOOP);
		m_fDrawRepeatLoop = FALSE;
		UpdateTitle();
	}
	else if (uID == ID_TIMER_SCROLLTITLE) {
		if (++m_nTitleWait >=  SCROLL_TITLE_INITWAIT) {
			if (m_pOptions->m_fScrollTitle) {
				if (!m_fDrawVolume && !m_fDrawRepeatLoop) {
					if (m_nTitleLen - SCROLL_TITLE_MARGIN > m_nTitleWidth) {
						m_nTitlePos += SCROLL_TITLE_INTERVAL;
						UpdateTitle();
					}
				}
			}
		}
	}
	else
		CMainWnd::OnTimer(uID);
}

void CKayMainWnd::RegisterClass()
{
	WNDCLASS	wc;
	memset(&wc, 0, sizeof(WNDCLASS));
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    wc.lpfnWndProc = (WNDPROC)KayMainWndProc;
    wc.hInstance = GetInst();
    wc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
    wc.lpszClassName = MAINWND_CLASS_NAME;
	::RegisterClass(&wc);
}

LRESULT CALLBACK CKayMainWnd::KayMainWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static CKayMainWnd* pMainWnd = NULL;

	switch (uMsg) {
	case WM_CREATE:
		CREATESTRUCT* pcs;
		pcs = (CREATESTRUCT*)lParam;
		pMainWnd = (CKayMainWnd*)pcs->lpCreateParams;
		return CMainWnd::MainWndProc(hwnd, uMsg, wParam, lParam);
	case WM_INITMENUPOPUP:
		pMainWnd->CKayMainWnd::OnInitMenuPopup((HMENU)wParam, LOWORD(lParam), HIWORD(lParam));//!!
		return CMainWnd::MainWndProc(hwnd, uMsg, wParam, lParam);
	}
	return CMainWnd::MainWndProc(hwnd, uMsg, wParam, lParam);
}

void CKayMainWnd::DrawTitle(HDC hDC)
{
	if (m_fDrawRepeatLoop) {
		DrawRepeatLoop(hDC);
		return;
	}
	else
		CMainWnd::DrawTitle(hDC);
}

void CKayMainWnd::DrawRepeatLoop(HDC hDC)
{
	SIZE size;
	TCHAR szTitle[MAX_PATH] = {0};
	RECT rc = {TITLE_LEFT, m_nTitleTop,
			TITLE_LEFT + m_nTitleWidth, m_nTitleTop + m_nTitleHeight};

	CTempStr str;

	if( (m_nRepeatLoopState&0x03) == 0x01 ) {
		str.LoadNormal(IDM_REPEAT_LOOP_SET_A);
	} else if( (m_nRepeatLoopState&0x03) == 0x02 ) {
		str.LoadNormal(IDM_REPEAT_LOOP_SET_B);
	} else if( (m_nRepeatLoopState&0x03) == 0x03 ) {
		str.LoadNormal(IDM_REPEAT_LOOP_PLAY);
	} else if( (m_nRepeatLoopState&0x03) == 0x00 ) {
		str.LoadNormal(IDM_REPEAT_LOOP_SET_CLEAR);
	}

	_tcscpy(szTitle, str);

	COLORREF crBk = SetTextColor(hDC, m_pSkin->GetColor(COLOR_TITLE));
	int nOld = SetBkMode(hDC, TRANSPARENT);
	HFONT hBk = (HFONT)SelectObject(hDC, m_hFntTitle);
	GetTextExtentPoint32(hDC, szTitle, _tcslen(szTitle), &size);
	DrawText(hDC, szTitle, -1, &rc, DT_LEFT | DT_BOTTOM | DT_SINGLELINE | DT_NOPREFIX);
	SetTextColor(hDC, crBk);
	SetBkMode(hDC, nOld);
	SelectObject(hDC, hBk);
}

void CKayMainWnd::OnPlayNextFolder()
{
	int nNext = GetNextFolderIndex();
	if (nNext == OPEN_NONE)
		return;

	BOOL fPlay = MAP_GetStatus(m_hMap) != MAP_STATUS_STOP;
	Open(nNext);
	if (fPlay)
		Play();
}

void CKayMainWnd::OnPlayPrevFolder()
{
	BOOL fPlay = MAP_GetStatus(m_hMap) != MAP_STATUS_STOP;

	int nPrev = GetPrevFolderIndex();
	if (nPrev == OPEN_NONE)
		return;

	Open(nPrev);
	if (fPlay)
		Play();
}

void CKayMainWnd::OnRepeatLoop(int iMenu)
{
//preJop:
	if(MAP_GetStatus(m_hMap)!=MAP_STATUS_PLAY)
		iMenu = -1;// ޴ --> 
	else
	if(IDM_REPEAT_LOOP == iMenu) {
		if( (m_nRepeatLoopState&0x01) == 0 ) {//A  ȵ 
			// ݺ  A
			iMenu = IDM_REPEAT_LOOP_SET_A;
		}
		else if( (m_nRepeatLoopState&0x02) == 0 ) {//B  ȵ 
			// ݺ  B
			iMenu = IDM_REPEAT_LOOP_SET_B;
		}
		else if( (m_nRepeatLoopState&0x03) == 0x03 ) {
			// ݺ  
			iMenu = IDM_REPEAT_LOOP_SET_CLEAR;
		}
	}
//mainJop:
	BOOL fDataChenged = FALSE;
	LONG lCurr = -1;
	switch(iMenu) {
	case IDM_REPEAT_LOOP_SET_A:
		{
			// ݺ  A -   ==>  
			m_nRepeatLoopState |= 0x01;
			m_fDrawRepeatLoop = TRUE;
			lCurr = m_lRepeatLoop_A = MAP_GetCurrent(m_hMap);
			fDataChenged = TRUE;
			break;
		}
	case IDM_REPEAT_LOOP_SET_B:
		{
			// ݺ  B -   ==>  
			m_nRepeatLoopState |= 0x02;
			m_fDrawRepeatLoop = TRUE;
			lCurr = m_lRepeatLoop_B = MAP_GetCurrent(m_hMap);
			fDataChenged = TRUE;
			break;
		}
	case IDM_REPEAT_LOOP_SET_CLEAR:
		{
			// ݺ   -  
			RepeatLoopReset(TRUE);
			break;
		}
	default:
		{
			// ݺ  
			RepeatLoopReset();
		}
	}

	if( (m_nRepeatLoopState&0x03) == 0x03 ) {
		// ݺ ƴϸ    Ƿ...
		if(fDataChenged) {
			//A  B  ٲǷ Start  End  Ѵ.
			if(m_lRepeatLoop_A<m_lRepeatLoop_B) {
				m_lRepeatLoopStart = m_lRepeatLoop_A;
				m_lRepeatLoopEnd   = m_lRepeatLoop_B;
			}
			else {
				m_lRepeatLoopStart = m_lRepeatLoop_B;
				m_lRepeatLoopEnd   = m_lRepeatLoop_A;
			}
		}

		//always Start <= End
		ASSERT(m_lRepeatLoopStart<m_lRepeatLoopEnd);
		//  ̸ ʳ .
		if(lCurr>=m_lRepeatLoopEnd)
			MAP_Seek(m_hMap, m_lRepeatLoopStart);
	}

	KillTimer(m_hWnd, ID_TIMER_REPEAT_LOOP);
	UpdateTitle();
	if(m_fDrawRepeatLoop) {
		SetTimer(m_hWnd, ID_TIMER_REPEAT_LOOP, TIMER_REPEAT_LOOP_INTERVAL, NULL);
	}
}

void CKayMainWnd::OnInitMenuPopup(HMENU hMenu, UINT uPos, BOOL fSystemMenu)
{
	TRACE0("CKayMainWnd::OnInitMenuPopup\n");
	if (uPos == IDM_PLAY) {
		CheckMenuItem(hMenu, IDM_REPEAT_LOOP_SET_A, ((m_nRepeatLoopState&0x01)==0x01)?MF_CHECKED:MF_UNCHECKED);
		CheckMenuItem(hMenu, IDM_REPEAT_LOOP_SET_B, ((m_nRepeatLoopState&0x02)==0x02)?MF_CHECKED:MF_UNCHECKED);
	}
	else if (uPos == IDM_TOOL) {
		CheckMenuItem(hMenu, IDM_TOOL_VOLMUTE, m_fVolMuted?MF_CHECKED:MF_UNCHECKED);
	}
}

static
void GetPath(TCHAR szIn[MAX_PATH], TCHAR szOut[MAX_PATH])
{
	_tcscpy(szOut, szIn);
	unsigned short *ptr = wcsrchr(szOut, _T('\\'));
	*ptr = _T('\0');
}

int CKayMainWnd::GetNextFolderIndex()
{
	if (m_nCurrent == OPEN_NONE)
		return OPEN_NONE;

	int nMax = m_pListFile->GetCount() - 1;

	TCHAR szCurPath[MAX_PATH];
	{
		// Get Current File Path
		FILEINFO *info = GetInfo(m_nCurrent);
		GetPath(info->szPath, szCurPath);
	}

	for (int i=m_nCurrent+1; i <= nMax; i++) {
		if (i == nMax) {
			return 0;
		}
		FILEINFO *tmp_info = GetInfo(i);
		TCHAR szTmpPath[MAX_PATH];
		GetPath(tmp_info->szPath, szTmpPath);

		if (wcscmp(szCurPath, szTmpPath)==0) continue;
		return i;
	}
	return 0;
}

int CKayMainWnd::GetPrevFolderIndex()
{
	if (m_nCurrent == OPEN_NONE)
		return OPEN_NONE;

	TCHAR szCurPath[MAX_PATH];
	{
		// Get Current File Path
		FILEINFO *info = GetInfo(m_nCurrent);
		GetPath(info->szPath, szCurPath);
	}

	int count=0;
	for (int i=m_nCurrent-1; i >= 0; i--) {
		FILEINFO *tmp_info = GetInfo(i);
		TCHAR szTmpPath[MAX_PATH];
		GetPath(tmp_info->szPath, szTmpPath);

		if (wcscmp(szCurPath, szTmpPath)==0) {
			continue;
		} else {
			count++;
			if (count==2) {
				return i+1;
			}
			_tcscpy(szCurPath, szTmpPath);
		}
	}
	return 0;
}

BOOL CKayMainWnd::Play(int nIndex)
{
	m_nRepeatLoopState = 0;
	return CMainWnd::Play(nIndex);
}

void CKayMainWnd::Seek(int nMilisec)
{
	if( ((m_nRepeatLoopState&0x03)==0x03) && (nMilisec>m_lRepeatLoopEnd) ) {
		MAP_Seek(m_hMap, m_lRepeatLoopStart);
	}
	else {
		MAP_Seek(m_hMap, nMilisec);
	}
}

void CKayMainWnd::OnPlayRew()
{
	Seek((MAP_GetCurrent(m_hMap) - m_pOptions->m_dwSkipInterval/*SKIP_INTERVAL*/));
}

void CKayMainWnd::OnPlayFf()
{
	Seek((MAP_GetCurrent(m_hMap) + m_pOptions->m_dwSkipInterval/*SKIP_INTERVAL*/));
}

void CKayMainWnd::OnToolVolUp()
{
	m_fVolMuted = FALSE;
	InitVolume();
	CMainWnd::OnToolVolUp();
}

void CKayMainWnd::OnToolVolDown()
{
	m_fVolMuted = FALSE;
	InitVolume();
	CMainWnd::OnToolVolDown();
}

void CKayMainWnd::DrawVolume(HDC hDC)
{
	SIZE size;
	TCHAR szTitle[MAX_PATH] = {0};
	RECT rc = {TITLE_LEFT, m_nTitleTop,
			TITLE_LEFT + m_nTitleWidth, m_nTitleTop + m_nTitleHeight};

	DWORD dwVolume;
	waveOutGetVolume(NULL, &dwVolume);
	dwVolume = max(LOWORD(dwVolume), HIWORD(dwVolume));
	dwVolume = dwVolume * 100 / 0xFFFF;
	wsprintf(szTitle, CTempStr(m_fVolMuted?IDS_FMT_DISP_VOLMUTE:IDS_FMT_DISP_VOLUME), dwVolume);//kayws426

	COLORREF crBk = SetTextColor(hDC, m_pSkin->GetColor(COLOR_TITLE));
	int nOld = SetBkMode(hDC, TRANSPARENT);
	HFONT hBk = (HFONT)SelectObject(hDC, m_hFntTitle);
	GetTextExtentPoint32(hDC, szTitle, _tcslen(szTitle), &size);
	DrawText(hDC, szTitle, -1, &rc, DT_LEFT | DT_BOTTOM | DT_SINGLELINE | DT_NOPREFIX);
	SetTextColor(hDC, crBk);
	SetBkMode(hDC, nOld);
	SelectObject(hDC, hBk);

#ifdef _WIN32_WCE_PPC
#define VOLBAR_LEFT		80
#define VOLBAR_WIDTH	125
#define VOLBAR_HEIGHT	8
#else
#define VOLBAR_LEFT		50
#define VOLBAR_WIDTH	145
#define VOLBAR_HEIGHT	8
#endif
	if (!dwVolume)
		return;

	int nWidth = VOLBAR_WIDTH * dwVolume / 100;
	RECT rcBar = {TITLE_LEFT + VOLBAR_LEFT, m_nTitleTop + m_nTitleHeight - size.cy + 1,
				TITLE_LEFT + VOLBAR_LEFT + nWidth, m_nTitleTop + m_nTitleHeight};
	HBRUSH hbr = CreateSolidBrush(m_pSkin->GetColor(COLOR_TITLE));
	FillRect(hDC, &rcBar, hbr);
	DeleteObject(hbr);
}
