// KayOptions.cpp: implementation of the CKayOptions class.
//
//////////////////////////////////////////////////////////////////////

#include "KayOptions.h"
#include "KayMainWnd.h"
//#include "KayPpcMainWnd.h"

//kayws426{{
#define NAME_USESYSTEMVOLUME	_T("UseSystemVolume")
#define NAME_RESUMEVOLUME		_T("ResumeVolume")
#define NAME_SKIP_INTERVAL		_T("SkipInterval")
#define NAME_LYRIC_WORDWRAP		_T("LyricWordWrap")
#define NAME_LYRIC_SHIFT_MSEC	_T("LyricShiftmsec")
#define NAME_LYRIC_WINDOW_POS	_T("LyricWndPos")
#define NAME_LYRIC_AUTOSHOW		_T("LyricAutoShow")
#define NAME_VOLUME_MUTE		_T("VolumeMute")
//kayws426}}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CKayOptions::CKayOptions()
{

}

CKayOptions::~CKayOptions()
{

}

void CKayOptions::Load(HANDLE hMap)
{
	__super::Load(hMap);

	HKEY hKey = 0;
	if (RegOpenKeyEx(HKEY_CURRENT_USER, KEY_SETTINGS,
						0, KEY_READ, &hKey) == ERROR_SUCCESS) {

		DWORD dwType, dwBuff, dwSize;
		dwSize = sizeof(dwBuff);

//kayws426{{
		if (RegQueryValueEx(hKey, NAME_USESYSTEMVOLUME, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS)
			m_fUseSystemVolume = dwBuff;
		if( RegQueryValueEx(hKey, NAME_RESUMEVOLUME, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS) {
			GetPlayerApp()->GetMainWnd()->m_dwCurrentSoftVolume = dwBuff;
			if( !m_fUseSystemVolume )
				GetPlayerApp()->GetMainWnd()->m_dwCurrentVolume = dwBuff;
		}
		if (RegQueryValueEx(hKey, NAME_SKIP_INTERVAL, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS)
			m_dwSkipInterval = dwBuff;
		if( RegQueryValueEx(hKey, NAME_VOLUME_MUTE, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS) {
			((CKayMainWnd*)GetPlayerApp()->GetMainWnd())->m_fVolMuted = dwBuff;
		}
		if (RegQueryValueEx(hKey, NAME_LYRIC_WINDOW_POS, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS)
			m_fLyricWndPos = dwBuff;
	//	if( RegQueryValueEx(hKey, NAME_LYRIC_AUTOSHOW, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS) {
	//		((CKayPpcMainWnd*)GetPlayerApp()->GetMainWnd())->m_bAutoShowLyricWnd = dwBuff;
	//	}
		if (RegQueryValueEx(hKey, NAME_LYRIC_SHIFT_MSEC, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS)
			m_iLyricShiftmsec = dwBuff;
		//Lyric Wnd
		if (RegQueryValueEx(hKey, NAME_LYRIC_WORDWRAP, 0, &dwType, (LPBYTE)&dwBuff, &dwSize) == ERROR_SUCCESS)
			m_fLyricWordWrap = dwBuff;
//kayws426}}
		RegCloseKey(hKey);
	}

	//kayws426{{
	MAP_SetVolume(hMap, GetPlayerApp()->GetMainWnd()->m_dwCurrentVolume);
	//kayws426}}
}

void CKayOptions::Save(HANDLE hMap)
{
	__super::Save(hMap);

	HKEY hKey = 0;
	DWORD dwDisposition;
	if (RegCreateKeyEx(HKEY_CURRENT_USER, KEY_SETTINGS, 0, NULL,
		REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, &dwDisposition) == ERROR_SUCCESS) {

//kayws426{{
		RegSetValueEx(hKey, NAME_USESYSTEMVOLUME, 0, REG_DWORD, (LPBYTE)&m_fUseSystemVolume, sizeof(DWORD));
		RegSetValueEx(hKey, NAME_RESUMEVOLUME, 0, REG_DWORD, (LPBYTE)&(GetPlayerApp()->GetMainWnd()->m_dwCurrentSoftVolume), sizeof(DWORD));
		RegSetValueEx(hKey, NAME_SKIP_INTERVAL, 0, REG_DWORD, (LPBYTE)&m_dwSkipInterval, sizeof(DWORD));
		RegSetValueEx(hKey, NAME_VOLUME_MUTE, 0, REG_DWORD, (LPBYTE)&(((CKayMainWnd*)GetPlayerApp()->GetMainWnd())->m_fVolMuted), sizeof(DWORD));
		RegSetValueEx(hKey, NAME_LYRIC_WINDOW_POS, 0, REG_DWORD, (LPBYTE)&m_fLyricWndPos, sizeof(DWORD));
		//RegSetValueEx(hKey, NAME_LYRIC_AUTOSHOW, 0, REG_DWORD, (LPBYTE)&(((CKayPpcMainWnd*)GetPlayerApp()->GetMainWnd())->m_bAutoShowLyricWnd), sizeof(DWORD));
		RegSetValueEx(hKey, NAME_LYRIC_SHIFT_MSEC, 0, REG_DWORD, (LPBYTE)&m_iLyricShiftmsec, sizeof(DWORD));
		//Lyric Wnd
		RegSetValueEx(hKey, NAME_LYRIC_WORDWRAP, 0, REG_DWORD, (LPBYTE)&m_fLyricWordWrap, sizeof(DWORD));
//kayws426}}

		RegCloseKey(hKey);
	}
}

static DWORD ff_secs[] = { 3000, 5000, 7000, 10000, 15000, 20000, 30000, 60000, -1 };
static DWORD lyric_shift_msecs[] = { 100, 200, 300, 400, 500, 700, 1000, 2000, 5000, -1 };

void CKayOptions::PlayerPageOnInitDialog(HWND hwndDlg)
{
	__super::PlayerPageOnInitDialog(hwndDlg);

//kayws426{{
	if (m_fUseSystemVolume)
		SendMessage(GetDlgItem(hwndDlg, IDC_CHECK_USESYSTEMVOLUME), BM_SETCHECK, 1, 0);

	{
		HWND hwndCmb = GetDlgItem(hwndDlg, IDC_CMB_FFREWSEC);
		int i = 0;
		bool sec_found = false;
		while (1) {
			if (ff_secs[i] == -1) break;
			TCHAR szSec[30];
			wsprintf(szSec, _T("%d"), ff_secs[i]/1000);
			SendMessage(hwndCmb, CB_ADDSTRING, 0, (LPARAM)szSec);
			if (ff_secs[i] == m_dwSkipInterval) {
				SendMessage(hwndCmb, CB_SETCURSEL, i, 0);
				sec_found = true;
			}
			i++;
		}
		if (sec_found == false) {
			TCHAR szSec[30];
			wsprintf(szSec, _T("%d"), m_dwSkipInterval/1000);
			SendMessage(hwndCmb, CB_ADDSTRING, 0, (LPARAM)szSec);
			SendMessage(hwndCmb, CB_SETCURSEL, i, 0);
		}
	}
	{
		HWND hwndCBLyricShiftmsec = GetDlgItem(hwndDlg, IDC_CMB_LYRIC_SHIFT_MSEC);
		BOOL found = FALSE;
		for(int i=0; lyric_shift_msecs[i]!=-1; i++) {
			TCHAR szMSec[30];
			wsprintf(szMSec, _T("%d"), lyric_shift_msecs[i]);
			SendMessage(hwndCBLyricShiftmsec, CB_ADDSTRING, 0, (LPARAM)szMSec);
			if (lyric_shift_msecs[i] == m_iLyricShiftmsec) {
				SendMessage(hwndCBLyricShiftmsec, CB_SETCURSEL, i, 0);
				found = TRUE;
			}
		}
		if (found == FALSE) {
			TCHAR szMSec[30];
			wsprintf(szMSec, _T("%d"), m_iLyricShiftmsec);
			SendMessage(hwndCBLyricShiftmsec, CB_ADDSTRING, 0, (LPARAM)szMSec);
			SendMessage(hwndCBLyricShiftmsec, CB_SETCURSEL, i, 0);
		}
	}
//kayws426}}
}

void CKayOptions::PlayerPageOnOK(HWND hwndDlg)
{
	__super::PlayerPageOnOK(hwndDlg);

//kayws426{{
	m_fUseSystemVolume = SendMessage(GetDlgItem(hwndDlg, IDC_CHECK_USESYSTEMVOLUME), BM_GETCHECK, 0, 0);

	{
//		HWND hwndCmb = GetDlgItem(hwndDlg, IDC_CMB_FFREWSEC);
//		int nIndex = SendMessage(hwndCmb, CB_GETCURSEL, 0, 0);
//		if (nIndex == CB_ERR) {
//			TRACE0("CB_ERR");
//		}
//		else {
//			TCHAR szSec[30];
//			SendMessage(hwndCmb, CB_GETLBTEXT, nIndex, (LPARAM)szSec);
//			TRACE(szSec);
//			m_dwSkipInterval = _wtoi(szSec) * 1000;
//		}

		HWND hwndCmb = GetDlgItem(hwndDlg, IDC_CMB_FFREWSEC);
		TCHAR szBuff[30];
		GetWindowText(hwndCmb, szBuff, 29);
		TRACE1("%d\n", szBuff);
		m_dwSkipInterval = _ttoi(szBuff) * 1000;
	}
	{
		HWND hwndCBLyricShiftmsec = GetDlgItem(hwndDlg, IDC_CMB_LYRIC_SHIFT_MSEC);
		TCHAR szBuff[30];
		GetWindowText(hwndCBLyricShiftmsec, szBuff, 29);
		TRACE1("%d\n", szBuff);
		m_iLyricShiftmsec = _ttoi(szBuff);
	}
//kayws426}}
}
